@@ -16,6 +16,13 @@
 #'   "Suggests". `NA` is shorthand for "Depends", "Imports" and "LinkingTo"
 #'   and is the default. `FALSE` is shorthand for no dependencies (i.e.
 #'   just check this package, not its dependencies).
+#'
+#'   The value "soft" means the same as `TRUE`, "hard" means the same as `NA`.
+#'
+#'   You can also specify dependencies from one or more additional fields,
+#'   common ones include:
+#'   - Config/Needs/website - for dependencies used in building the pkgdown site.
+#'   - Config/Needs/coverage for dependencies used in calculating test coverage.
 #' @param quiet If `TRUE`, suppress output.
 #' @param upgrade One of "default", "ask", "always", or "never". "default"
 #'   respects the value of the `R_REMOTES_UPGRADE` environment variable if set,
@@ -131,12 +138,16 @@ dev_package_deps <- function(pkgdir = ".", dependencies = NA,
       repos[missing_repos] <- bioc_repos[missing_repos]
   }
 
-  combine_deps(
-    package_deps(deps, repos = repos, type = type),
-    remote_deps(pkg))
+  cran_deps <- package_deps(deps, repos = repos, type = type)
+
+  res <- combine_remote_deps(cran_deps, extra_deps(pkg, "remotes"))
+
+  res <- do.call(rbind, c(list(res), lapply(get_extra_deps(pkg, dependencies), extra_deps, pkg = pkg), stringsAsFactors = FALSE))
+
+  res[!duplicated(res$package, fromLast = TRUE), ]
 }
 
-combine_deps <- function(cran_deps, remote_deps) {
+combine_remote_deps <- function(cran_deps, remote_deps) {
   # If there are no dependencies there will be no remote dependencies either,
   # so just return them (and don't force the remote_deps promise)
   if (nrow(cran_deps) == 0) {
@@ -189,8 +200,17 @@ compare_versions <- function(inst, remote, is_cran) {
     integer(1))
 }
 
-has_dev_remotes <- function(pkg) {
-  !is.null(pkg[["remotes"]])
+has_extra_deps <- function(pkg, dependencies) {
+  any(dependencies %in% names(pkg))
+}
+
+get_extra_deps <- function(pkg, dependencies) {
+  dependencies <- tolower(dependencies)
+
+  dependencies <- intersect(dependencies, names(pkg))
+
+  #remove standard dependencies
+  setdiff(dependencies, tolower(standardise_dep(TRUE)))
 }
 
 #' @export
@@ -248,6 +268,7 @@ update.package_deps <- function(object,
                            type = getOption("pkgType"),
                            ...) {
 
+  dependencies <- standardise_dep(dependencies)
 
   object <- upgradable_packages(object, upgrade, quiet)
 
@@ -379,7 +400,9 @@ find_deps <- function(packages, available = available_packages(),
     rec_flat <- character()
   }
 
-  unique(c(if (include_pkgs) packages, top_flat, rec_flat))
+  # We need to return these in reverse order, so that the packages furthest
+  # down in the tree are installed first.
+  unique(rev(c(if (include_pkgs) packages, top_flat, rec_flat)))
 }
 
 #' Standardise dependencies using the same logical as [install.packages]
@@ -392,6 +415,11 @@ find_deps <- function(packages, available = available_packages(),
 #'   "Suggests". `NA` is shorthand for "Depends", "Imports" and "LinkingTo"
 #'   and is the default. `FALSE` is shorthand for no dependencies.
 #'
+#'   The value "soft" means the same as `TRUE`, "hard" means the same as `NA`.
+#'
+#'   Any additional values that don't match one of the standard dependency
+#'   types are filtered out.
+#'
 #' @seealso <http://r-pkgs.had.co.nz/description.html#dependencies> for
 #' additional information on what each dependency type means.
 #' @keywords internal
@@ -404,7 +432,13 @@ standardise_dep <- function(x) {
   } else if (identical(x, FALSE)) {
     character(0)
   } else if (is.character(x)) {
-    x
+    if (any(x == "hard")) {
+      c("Depends", "Imports", "LinkingTo")
+    } else if (any(x == "soft")) {
+      c("Depends", "Imports", "LinkingTo", "Suggests")
+    } else {
+      intersect(x, c("Depends", "Imports", "LinkingTo", "Suggests", "Enhances"))
+    }
   } else {
     stop("Dependencies must be a boolean or a character vector", call. = FALSE)
   }
@@ -416,6 +450,7 @@ standardise_dep <- function(x) {
 #' that are already installed, and also upgrades out dated dependencies.
 #'
 #' @param packages Character vector of packages to update.
+#' @param force Deprecated, this argument has no effect.
 #' @inheritParams install_github
 #' @seealso [package_deps()] to see which packages are out of date/
 #'   missing.
@@ -436,6 +471,11 @@ update_packages <- function(packages = TRUE,
                             repos = getOption("repos"),
                             type = getOption("pkgType"),
                             ...) {
+
+  if (isTRUE(force)) {
+    .Deprecated(msg = "`update_packages(force = TRUE)` is deprecated and has no effect.")
+  }
+
   if (isTRUE(packages)) {
     packages <- utils::installed.packages()[, "Package"]
   }
@@ -444,7 +484,7 @@ update_packages <- function(packages = TRUE,
   update(pkgs,
          dependencies = dependencies,
          upgrade = upgrade,
-         force = force,
+         force = FALSE,
          quiet = quiet,
          build = build,
          build_opts = build_opts,
@@ -476,11 +516,15 @@ fix_repositories <- function(repos) {
   repos
 }
 
-parse_one_remote <- function(x, ...) {
+parse_one_extra <- function(x, ...) {
   pieces <- strsplit(x, "::", fixed = TRUE)[[1]]
 
   if (length(pieces) == 1) {
-    type <- "github"
+    if (!grepl("/", pieces)) {
+      type <- "cran"
+    } else {
+      type <- "github"
+    }
     repo <- pieces
   } else if (length(pieces) == 2) {
     type <- pieces[1]
@@ -493,8 +537,7 @@ parse_one_remote <- function(x, ...) {
     # `asNamespace("remotes")` because when used as a script in
     # install-github.R there is no remotes namespace.
 
-    fun <- get(paste0(tolower(type), "_remote"),
-      envir = environment(sys.function()), mode = "function", inherits = FALSE)
+    fun <- get(paste0(tolower(type), "_remote"), mode = "function", inherits = TRUE)
 
     res <- fun(repo, ...)
     }, error = function(e) stop("Unknown remote type: ", type, "\n  ", conditionMessage(e), call. = FALSE)
@@ -502,10 +545,10 @@ parse_one_remote <- function(x, ...) {
   res
 }
 
-split_remotes <- function(x) {
+split_extra_deps <- function(x, name = "Remotes") {
   pkgs <- trim_ws(unlist(strsplit(x, ",[[:space:]]*")))
   if (any((res <- grep("[[:space:]]+", pkgs)) != -1)) {
-    stop("Missing commas separating Remotes: '", pkgs[res], "'", call. = FALSE)
+    stop("Missing commas separating ", name, ": '", pkgs[res], "'", call. = FALSE)
   }
   pkgs
 }
@@ -524,22 +567,22 @@ package_deps_new <- function(package = character(), installed = character(),
   res
 }
 
-remote_deps <- function(pkg) {
-  if (!has_dev_remotes(pkg)) {
+extra_deps <- function(pkg, field) {
+  if (!has_extra_deps(pkg, field)) {
     return(package_deps_new())
   }
+  dev_packages <- split_extra_deps(pkg[[field]])
+  extra <- lapply(dev_packages, parse_one_extra)
 
-  dev_packages <- split_remotes(pkg[["remotes"]])
-  remote <- lapply(dev_packages, parse_one_remote)
-
-  package <- vapply(remote, function(x) remote_package_name(x), character(1), USE.NAMES = FALSE)
+  package <- vapply(extra, function(x) remote_package_name(x), character(1), USE.NAMES = FALSE)
   installed <- vapply(package, function(x) local_sha(x), character(1), USE.NAMES = FALSE)
-  available <- vapply(remote, function(x) remote_sha(x), character(1), USE.NAMES = FALSE)
+  available <- vapply(extra, function(x) remote_sha(x), character(1), USE.NAMES = FALSE)
   diff <- installed == available
   diff <- ifelse(!is.na(diff) & diff, CURRENT, BEHIND)
   diff[is.na(installed)] <- UNINSTALLED
+  is_cran_remote <- vapply(extra, inherits, logical(1), "cran_remote")
 
-  package_deps_new(package, installed, available, diff, is_cran = FALSE, remote)
+  package_deps_new(package, installed, available, diff, is_cran = is_cran_remote, extra)
 }
 
 
@@ -590,7 +633,7 @@ upgradable_packages <- function(x, upgrade, quiet, is_interactive = interactive(
         choices <- c("All", "CRAN packages only", "None", choices)
       }
 
-      res <- select_menu(choices, title = "These packages have more recent versions available.\nWhich would you like to update?")
+      res <- select_menu(choices, title = "These packages have more recent versions available.\nIt is recommended to update all of them.\nWhich would you like to update?")
 
       if ("None" %in% res || length(res) == 0) {
         return(x[uninstalled, ])
@@ -622,8 +665,7 @@ select_menu <- function(choices, title = NULL, msg = "Enter one or more numbers,
   fop <- format(op)
   cat("", fop, "", sep = "\n")
   repeat {
-    cat(msg, "\n", sep = "")
-    answer <- readLines(n = 1)
+    answer <- readline(msg)
     answer <- strsplit(answer, "[ ,]+")[[1]]
     if (all(answer %in% seq_along(choices))) {
       return(choices[as.integer(answer)])
